@@ -35,7 +35,10 @@ arch_archive_location_file (t_uchar * archive_name)
 
   if (! arch_valid_archive_name(archive_name)) 
     {
-      panic("Invalid archive name `%s'.");
+      int ign;
+
+      printfmt (&ign, 2, "Invalid archive name `%s'.", archive_name);
+      exit (2);
     }
 
   location_dir = arch_my_archive_locations_dir ();
@@ -58,13 +61,19 @@ arch_set_archive_location (t_uchar * archive_name, t_uchar * location, int force
   dir = file_name_directory_file (0, file);
   tmp_name = tmp_file_name (dir, ",,new-location");
 
-
   if (!force_p && !safe_access (file, F_OK))
     {
       if (! quietly_fail)
         {
           safe_printfmt (2, "archive already registered: %s\n", archive_name);
-          exit (2);
+          if (quietly_fail == ARCH_REG_FAIL_NOFAIL)
+            {
+              return;
+            }
+          else
+            {
+              exit (2);
+            }
         }
     }
   else
