@@ -13,6 +13,7 @@
 #include "hackerlab/fs/file-names.h"
 #include "hackerlab/vu/safe.h"
 #include "hackerlab/arrays/ar.h"
+#include "hackerlab/char/pika-escaping-utils.h"
 #include "tla/libfsutils/read-line.h"
 #include "tla/libawk/relassoc.h"
 #include "tla/libfsutils/find-utils.h"
@@ -25,14 +26,23 @@
 
 /* __STDC__ prototypes for static functions */
 static void fix_mod_locs (assoc_table loc_of, rel_table table);
+static void arch_print_file_metadata_diffs (int out_fd,
+                                            struct arch_changeset_report * report,
+                                            int escape_classes);
+static void arch_print_dir_metadata_diffs (int out_fd,
+                                           struct arch_changeset_report * report,
+                                           int escape_classes);
+static void arch_print_file_diffs (int out_fd,
+                                   struct arch_changeset_report * report,
+                                   int escape_classes);
+static void arch_print_added_file_diffs (int out_fd,
+                                         struct arch_changeset_report * report,
+                                         int escape_classes);
+static char * no_dot (char *name);
 static void report_unique_files_and_symlinks (rel_table * files, rel_table * symlinks,
                                               rel_table index,
                                               t_uchar * archive_dir, t_uchar * archive_basename);
-static void find_renames (rel_table * out, rel_table common, assoc_table orig_dir_tag_of, assoc_table mod_dir_tag_of);
-static void arch_print_file_metadata_diffs (int out_fd, struct arch_changeset_report * report);
-static void arch_print_dir_metadata_diffs (int out_fd, struct arch_changeset_report * report);
-static void arch_print_file_diffs (int out_fd, struct arch_changeset_report * report);
-static void arch_print_added_file_diffs (int out_fd, struct arch_changeset_report * report);
+static void find_renames (rel_table * out, rel_table common, assoc_table orig_dir_id_of, assoc_table mod_dir_id_of);
 
 
 
@@ -87,8 +97,6 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
   assoc_table mod_file_loc_of = 0;
   assoc_table mod_file_id_of = 0;
 
-  struct rel_join_output_spec *output_spec = 0;
-
 
   orig_files_index_path = file_name_in_vicinity (0, path, "orig-files-index");
   orig_dirs_index_path = file_name_in_vicinity (0, path, "orig-dirs-index");
@@ -143,21 +151,13 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
     mod_dirs_by_name = rel_copy_table (report->mod_dirs_index);
     rel_sort_table_by_field (0, mod_dirs_by_name, 0);
 
-    output_spec = rel_join_output (1,0, 1,1, -1);
-    removed_dirs_index = rel_join (1, output_spec, 1, 1, report->orig_dirs_index, report->mod_dirs_index);
-    ar_free ((void **)&output_spec, 0);
-    output_spec = rel_join_output (2,0, 2,1, -1);
-    added_dirs_index = rel_join (2, output_spec, 1, 1, report->orig_dirs_index, report->mod_dirs_index);
-    ar_free ((void **)&output_spec, 0);
+    removed_dirs_index = rel_join (1, rel_join_output (1,0, 1,1, -1), 1, 1, report->orig_dirs_index, report->mod_dirs_index);
+    added_dirs_index = rel_join (2, rel_join_output (2,0, 2,1, -1), 1, 1, report->orig_dirs_index, report->mod_dirs_index);
 
     t = rel_copy_table (removed_dirs_index);
     rel_sort_table_by_field (0, t, 0);
-    output_spec = rel_join_output (1,0, 1,1, 2,1, -1);
-    report->removed_dirs = rel_join (-1, output_spec, 0, 0, t, original_only_dir_metadata);
-    ar_free ((void **)&output_spec, 0);
-    output_spec = rel_join_output (1,0, 1,1, 1,0, -1);
-    t2 = rel_join (1, output_spec, 0, 0, t, original_only_dir_metadata);
-    ar_free ((void **)&output_spec, 0);
+    report->removed_dirs = rel_join (-1, rel_join_output (1,0, 1,1, 2,1, -1), 0, 0, t, original_only_dir_metadata);
+    t2 = rel_join (1, rel_join_output (1,0, 1,1, 1,0, -1), 0, 0, t, original_only_dir_metadata);
     rel_append_x (&report->removed_dirs, t2);
     rel_sort_table_by_field (0, report->removed_dirs, 0);
     rel_free_table (t);
@@ -166,12 +166,8 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
 
     t = rel_copy_table (added_dirs_index);
     rel_sort_table_by_field (0, t, 0);
-    output_spec = rel_join_output (1,0, 1,1, 2,1, -1);
-    report->added_dirs = rel_join (-1, output_spec, 0, 0, t, modified_only_dir_metadata);
-    ar_free ((void **)&output_spec, 0);
-    output_spec = rel_join_output (1,0, 1,1, 1,0, -1);
-    t2 = rel_join (1, output_spec, 0, 0, t, modified_only_dir_metadata);
-    ar_free ((void **)&output_spec, 0);
+    report->added_dirs = rel_join (-1, rel_join_output (1,0, 1,1, 2,1, -1), 0, 0, t, modified_only_dir_metadata);
+    t2 = rel_join (1, rel_join_output (1,0, 1,1, 1,0, -1), 0, 0, t, modified_only_dir_metadata);
     rel_append_x (&report->added_dirs, t2);
     rel_sort_table_by_field (0, report->added_dirs, 0);
     rel_free_table (t);
@@ -190,12 +186,8 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
    * Removed and added filenames and symlinks
    */
 
-  output_spec = rel_join_output (1,0, 1,1, -1);
-  removed_files_and_symlinks_index = rel_join (1, output_spec, 1, 1, report->orig_files_index, report->mod_files_index);
-  ar_free ((void **)&output_spec, 0);
-  output_spec = rel_join_output (2,0, 2,1, -1);
-  added_files_and_symlinks_index = rel_join (2, output_spec, 1, 1, report->orig_files_index, report->mod_files_index);
-  ar_free ((void **)&output_spec, 0);
+  removed_files_and_symlinks_index = rel_join (1, rel_join_output (1,0, 1,1, -1), 1, 1, report->orig_files_index, report->mod_files_index);
+  added_files_and_symlinks_index = rel_join (2, rel_join_output (2,0, 2,1, -1), 1, 1, report->orig_files_index, report->mod_files_index);
 
   report_unique_files_and_symlinks (&report->removed_files, &report->removed_symlinks,
                                     removed_files_and_symlinks_index,
@@ -208,12 +200,8 @@ arch_evaluate_changeset (struct arch_changeset_report * report, t_uchar * path)
   /****************************************************************
    * renamed files and dirs
    */
-  output_spec = rel_join_output (1,0, 2,0, 1,1, -1);
-  common_dirs_index = rel_join (-1, output_spec, 1, 1, report->orig_dirs_index, report->mod_dirs_index);
-  ar_free ((void **)&output_spec, 0);
-  output_spec = rel_join_output (1,0, 2,0, 1,1, -1);
-  common_files_index = rel_join (-1, output_spec, 1, 1, report->orig_files_index, report->mod_files_index);
-  ar_free ((void **)&output_spec, 0);
+  common_dirs_index = rel_join (-1, rel_join_output (1,0, 2,0, 1,1, -1), 1, 1, report->orig_dirs_index, report->mod_dirs_index);
+  common_files_index = rel_join (-1, rel_join_output (1,0, 2,0, 1,1, -1), 1, 1, report->orig_files_index, report->mod_files_index);
 
   find_renames (&report->renamed_dirs, common_dirs_index, orig_dir_id_of, mod_dir_id_of);
   find_renames (&report->renamed_files, common_files_index, orig_dir_id_of, mod_dir_id_of);
@@ -440,7 +428,8 @@ fix_mod_locs (assoc_table loc_of, rel_table table)
 
 static void
 arch_print_file_metadata_diffs (int out_fd,
-                                struct arch_changeset_report * report)
+                                struct arch_changeset_report * report,
+                                int escape_classes)
 {
   int x;
 
@@ -450,8 +439,11 @@ arch_print_file_metadata_diffs (int out_fd,
       t_uchar * mod_meta_path = 0;
       t_uchar * orig_meta = 0;
       t_uchar * mod_meta = 0;
+      t_uchar * file_metadata_changed = 0;
 
-      safe_printfmt (out_fd, "    %s\n", report->file_metadata_changed[x][0]);
+      file_metadata_changed = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                          report->file_metadata_changed[x][0]);
+      safe_printfmt (out_fd, "    %s\n", file_metadata_changed);
 
       orig_meta_path = str_alloc_cat (0, report->file_metadata_changed[x][2], ".meta-orig");
       mod_meta_path = str_alloc_cat (0, report->file_metadata_changed[x][2], ".meta-mod");
@@ -460,6 +452,7 @@ arch_print_file_metadata_diffs (int out_fd,
 
       safe_printfmt (out_fd, "        %s\n        => %s\n", orig_meta, mod_meta);
 
+      lim_free (0, file_metadata_changed);
       lim_free (0, orig_meta_path);
       lim_free (0, mod_meta_path);
       lim_free (0, orig_meta);
@@ -469,7 +462,8 @@ arch_print_file_metadata_diffs (int out_fd,
 
 static void
 arch_print_dir_metadata_diffs (int out_fd,
-                               struct arch_changeset_report * report)
+                               struct arch_changeset_report * report,
+                               int escape_classes)
 {
   int x;
 
@@ -479,8 +473,11 @@ arch_print_dir_metadata_diffs (int out_fd,
       t_uchar * mod_meta_path = 0;
       t_uchar * orig_meta = 0;
       t_uchar * mod_meta = 0;
+      t_uchar * dir_metadata_changed = 0;
 
-      safe_printfmt (out_fd, "    %s\n", report->dir_metadata_changed[x][0]);
+      dir_metadata_changed = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                         report->dir_metadata_changed[x][0]);
+      safe_printfmt (out_fd, "    %s\n", dir_metadata_changed);
 
       orig_meta_path = file_name_in_vicinity (0, report->dir_metadata_changed[x][2], "=dir-meta-orig");
       mod_meta_path = file_name_in_vicinity (0, report->dir_metadata_changed[x][2], "=dir-meta-mod");
@@ -489,6 +486,7 @@ arch_print_dir_metadata_diffs (int out_fd,
 
       safe_printfmt (out_fd, "        %s\n        => %s\n", orig_meta, mod_meta);
 
+      lim_free (0, dir_metadata_changed);
       lim_free (0, orig_meta_path);
       lim_free (0, mod_meta_path);
       lim_free (0, orig_meta);
@@ -498,7 +496,8 @@ arch_print_dir_metadata_diffs (int out_fd,
 
 static void
 arch_print_file_diffs (int out_fd,
-                       struct arch_changeset_report * report)
+                       struct arch_changeset_report * report,
+                       int escape_classes)
 {
   int x;
 
@@ -518,7 +517,8 @@ arch_print_file_diffs (int out_fd,
 
 static void
 arch_print_added_file_diffs (int out_fd,
-                             struct arch_changeset_report * report)
+                             struct arch_changeset_report * report,
+                             int escape_classes)
 {
   int x;
 
@@ -529,32 +529,32 @@ arch_print_added_file_diffs (int out_fd,
 }
 
 void
-arch_print_changeset_diffs (int out_fd, struct arch_changeset_report * report)
+arch_print_changeset_diffs (int out_fd, struct arch_changeset_report * report, int escape_classes)
 {
   if (report->file_metadata_changed)
     {
       safe_printfmt (out_fd, "* file metadata changed\n\n");
-      arch_print_file_metadata_diffs (out_fd, report);
+      arch_print_file_metadata_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
 
   if (report->dir_metadata_changed)
     {
       safe_printfmt (out_fd, "* dir metadata changed\n\n");
-      arch_print_dir_metadata_diffs (out_fd, report);
+      arch_print_dir_metadata_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
 
   if (report->patched_regular_files)
     {
       safe_printfmt (out_fd, "* modified files\n\n");
-      arch_print_file_diffs (out_fd, report);
+      arch_print_file_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
   if (report->added_files)
     {
       safe_printfmt (out_fd, "* added files\n\n");
-      arch_print_added_file_diffs (out_fd, report);
+      arch_print_added_file_diffs (out_fd, report, escape_classes);
       safe_printfmt (out_fd, "\n");
     }
 }
@@ -570,7 +570,7 @@ no_dot (char *name)
 }
 
 void
-arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int diffs)
+arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int diffs, int escape_classes)
 {
   int x;
 
@@ -578,14 +578,26 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
     {
       safe_printfmt (out_fd, "* removed directories\n\n");
       for (x = 0; x < rel_n_records (report->removed_dirs); ++x)
-        safe_printfmt (out_fd, "    %s\n", no_dot(report->removed_dirs[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->removed_dirs[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
   if (report->added_dirs)
     {
       safe_printfmt (out_fd, "* added directories\n\n");
       for (x = 0; x < rel_n_records (report->added_dirs); ++x)
-        safe_printfmt (out_fd, "    %s\n", no_dot(report->added_dirs[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->added_dirs[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
 
@@ -593,14 +605,26 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
     {
       safe_printfmt (out_fd, "* removed files\n\n");
       for (x = 0; x < rel_n_records (report->removed_files); ++x)
-        safe_printfmt (out_fd, "    %s\n", no_dot(report->removed_files[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->removed_files[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
   if (report->added_files)
     {
       safe_printfmt (out_fd, "* added files\n\n");
       for (x = 0; x < rel_n_records (report->added_files); ++x)
-        safe_printfmt (out_fd, "    %s\n", no_dot(report->added_files[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->added_files[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
 
@@ -608,16 +632,26 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
     {
       safe_printfmt (out_fd, "* removed symlinks\n\n");
       for (x = 0; x < rel_n_records (report->removed_symlinks); ++x)
-        safe_printfmt (out_fd, "    %s\n",
-                       no_dot(report->removed_symlinks[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->removed_symlinks[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
   if (report->added_symlinks)
     {
       safe_printfmt (out_fd, "* added symlinks\n\n");
       for (x = 0; x < rel_n_records (report->added_symlinks); ++x)
-        safe_printfmt (out_fd, "    %s\n",
-                       no_dot (report->added_symlinks[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->added_symlinks[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
 
@@ -626,9 +660,15 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
       safe_printfmt (out_fd, "* renamed files and symlinks\n\n");
       for (x = 0; x < rel_n_records (report->renamed_files); ++x)
         {
-          safe_printfmt (out_fd, "    %s\n     => %s\n\n",
-                         no_dot (report->renamed_files[x][0]),
-                         no_dot (report->renamed_files[x][1]));
+          t_uchar * item1;
+          t_uchar * item2;
+          item1 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                              report->renamed_files[x][0]);
+          item2 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                              report->renamed_files[x][1]);
+          safe_printfmt (out_fd, "    %s\n     => %s\n\n", no_dot (item1), no_dot (item2));
+          lim_free (0, item2);
+          lim_free (0, item1);
         }
     }
   if (report->renamed_dirs)
@@ -636,9 +676,15 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
       safe_printfmt (out_fd, "* renamed directories\n\n");
       for (x = 0; x < rel_n_records (report->renamed_dirs); ++x)
         {
-          safe_printfmt (out_fd, "    %s\n     => %s\n\n",
-                         no_dot (report->renamed_dirs[x][0]),
-                         no_dot (report->renamed_dirs[x][1]));
+          t_uchar * item1;
+          t_uchar * item2;
+          item1 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                              report->renamed_dirs[x][0]);
+          item2 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                              report->renamed_dirs[x][1]);
+          safe_printfmt (out_fd, "    %s\n     => %s\n\n", no_dot (item1), no_dot (item2));
+          lim_free (0, item2);
+          lim_free (0, item1);
         }
     }
 
@@ -647,8 +693,11 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
       safe_printfmt (out_fd, "* files replaced by symlinks \n\n");
       for (x = 0; x < rel_n_records (report->file_to_symlink); ++x)
         {
-          safe_printfmt (out_fd, "    %s\n",
-                         no_dot (report->file_to_symlink[x][0]));
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->file_to_symlink[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
@@ -657,8 +706,11 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
       safe_printfmt (out_fd, "* symlinks replaced by files\n\n");
       for (x = 0; x < rel_n_records (report->symlink_to_file); ++x)
         {
-          safe_printfmt (out_fd, "    %s\n",
-                         no_dot (report->symlink_to_file[x][0]));
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->symlink_to_file[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
         }
       safe_printfmt (out_fd, "\n");
     }
@@ -669,14 +721,17 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
 
       if (diffs)
         {
-          arch_print_file_metadata_diffs (out_fd, report);
+          arch_print_file_metadata_diffs (out_fd, report, escape_classes);
         }
       else
         {
           for (x = 0; x < rel_n_records (report->file_metadata_changed); ++x)
             {
-              safe_printfmt (out_fd, "    %s\n",
-                             no_dot (report->file_metadata_changed[x][0]));
+              t_uchar * item;
+              item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                 report->file_metadata_changed[x][0]);
+              safe_printfmt (out_fd, "    %s\n", no_dot (item));
+              lim_free (0, item);
             }
         }
       safe_printfmt (out_fd, "\n");
@@ -688,14 +743,17 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
 
       if (diffs)
         {
-          arch_print_file_metadata_diffs (out_fd, report);
+          arch_print_dir_metadata_diffs (out_fd, report, escape_classes);
         }
       else
         {
           for (x = 0; x < rel_n_records (report->dir_metadata_changed); ++x)
             {
-              safe_printfmt (out_fd, "    %s\n",
-                             no_dot (report->dir_metadata_changed[x][0]));
+              t_uchar * item;
+              item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                 report->dir_metadata_changed[x][0]);
+              safe_printfmt (out_fd, "    %s\n", no_dot (item));
+              lim_free (0, item);
             }
         }
       safe_printfmt (out_fd, "\n");
@@ -705,16 +763,26 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
     {
       safe_printfmt (out_fd, "* retargeted symlinks\n\n");
       for (x = 0; x < rel_n_records (report->patched_symlinks); ++x)
-        safe_printfmt (out_fd, "    %s\n",
-                       no_dot (report->patched_symlinks[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->patched_symlinks[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
   if (report->patched_binaries)
     {
       safe_printfmt (out_fd, "* modified binary files\n\n");
       for (x = 0; x < rel_n_records (report->patched_binaries); ++x)
-        safe_printfmt (out_fd, "    %s\n",
-                       no_dot (report->patched_binaries[x][0]));
+        {
+          t_uchar * item;
+          item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                             report->patched_binaries[x][0]);
+          safe_printfmt (out_fd, "    %s\n", no_dot (item));
+          lim_free (0, item);
+        }
       safe_printfmt (out_fd, "\n");
     }
   if (report->patched_regular_files)
@@ -723,14 +791,17 @@ arch_print_changeset  (int out_fd, struct arch_changeset_report * report, int di
 
       if (diffs)
         {
-          arch_print_file_diffs (out_fd, report);
+          arch_print_file_diffs (out_fd, report, escape_classes);
         }
       else
         {
           for (x = 0; x < rel_n_records (report->patched_regular_files); ++x)
             {
-              safe_printfmt (out_fd, "    %s\n",
-                             no_dot (report->patched_regular_files[x][0]));
+              t_uchar * item;
+              item = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                 report->patched_regular_files[x][0]);
+              safe_printfmt (out_fd, "    %s\n", no_dot (item));
+              lim_free (0, item);
             }
         }
       safe_printfmt (out_fd, "\n");
