@@ -22,6 +22,7 @@
 #include "tla/libarch/archive.h"
 #include "tla/libarch/archives.h"
 #include "tla/libarch/patch-logs.h"
+#include "tla/libarch/cmdutils.h"
 #include "tla/libarch/cmd-ancestry.h"
 
 
@@ -198,7 +199,7 @@ arch_cmd_ancestry (t_uchar * program_name, int argc, char * argv[])
       {
         t_uchar * package = 0;
         rel_table versions = 0;
-
+        arch_check_for (arch, arch_req_package, revision_spec);
         package = arch_parse_package_name (arch_ret_package, 0, revision_spec);
         versions = arch_archive_versions (arch, package);
         arch_sort_table_by_name_field (1, versions, 0);
@@ -220,7 +221,8 @@ arch_cmd_ancestry (t_uchar * program_name, int argc, char * argv[])
     if (arch_valid_package_name (revision_spec, arch_maybe_archive, arch_req_version, 0))
       {
         rel_table revisions = 0;
-
+        
+        arch_check_for (arch, arch_req_version, revision_spec);
         version = arch_parse_package_name (arch_ret_package_version, 0, revision_spec);
 
         revisions = arch_archive_revisions (arch, version, 1);
@@ -238,6 +240,7 @@ arch_cmd_ancestry (t_uchar * program_name, int argc, char * argv[])
       }
     else
       {
+        arch_check_for (arch, arch_req_patch_level, revision_spec);
         version = arch_parse_package_name (arch_ret_package_version, 0, revision_spec);
         revision = arch_parse_package_name (arch_ret_non_archive, 0, revision_spec);
       }
@@ -343,6 +346,8 @@ arch_cmd_ancestry (t_uchar * program_name, int argc, char * argv[])
 
                 safe_printfmt (1, " ... %s\n", level);
               }
+            else
+              safe_printfmt (1, "\n");
 
             safe_flush (1);
 
