@@ -45,7 +45,7 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
   OP (opt_noremote, "R", "exclude-remote", 0, \
       "Exclude MIRROR and SOURCE archives.") 
 
-t_uchar arch_cmd_archives_help[] = ("Report registered archives and their locations.\n"
+t_uchar arch_cmd_archives_help[] = ("report registered archives and their locations\n"
 				    "Print a list of registered archives and their locations\n"
 				    "\n"
 				    "If [search regex] is given then only archives with names\n"
