@@ -45,7 +45,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "Display a release identifier string\n" \
       "and exit.") \
   OP (opt_file_list, 0, "file-list FILES", 1, \
-      "record only diffs of selected files")
+      "record only diffs of selected files") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 t_uchar arch_cmd_changeset_help[] = ("compute a whole-tree changeset\n"
                                      "Create the output directory DESTINATION (it must not already\n"
@@ -75,6 +77,7 @@ arch_cmd_changeset (t_uchar * program_name, int argc, char * argv[])
   int o;
   struct opt_parsed * option;
   t_uchar * file_list_file = 0;
+  int escape_classes = arch_escape_classes;
 
   safe_buffer_fd (1, 0, O_WRONLY, 0);
 
@@ -106,6 +109,12 @@ arch_cmd_changeset (t_uchar * program_name, int argc, char * argv[])
             file_list_file = str_save (0, option->arg_string);
             break;
           }
+
+	case opt_unescaped:
+	  {
+	    escape_classes = 0;
+	    break;
+	  }
         }
     }
 
@@ -121,7 +130,7 @@ arch_cmd_changeset (t_uchar * program_name, int argc, char * argv[])
 
     if (argc == 4 && !file_list_file)
       {
-        arch_make_changeset (&report, argv[1], argv[2], argv[3], arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, 0, 0);
+        arch_make_changeset (&report, argv[1], argv[2], argv[3], arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, 0, 0, escape_classes);
       }
     else
       {
@@ -145,7 +154,7 @@ arch_cmd_changeset (t_uchar * program_name, int argc, char * argv[])
             safe_close (in_fd);
           }
 
-        arch_make_files_changeset (&report, argv[3], file_list, argv[1], argv[2], arch_unspecified_id_tagging, arch_inventory_unrecognized);
+        arch_make_files_changeset (&report, argv[3], file_list, argv[1], argv[2], arch_unspecified_id_tagging, arch_inventory_unrecognized, escape_classes);
 
         rel_free_table (file_list);
       }
