@@ -69,6 +69,8 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "commit even if out of date") \
   OP (opt_file_list, 0, "file-list FILE", 1, \
       "commit only changes to files listed in FILE") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 
 t_uchar arch_cmd_commit_help[] =
@@ -112,6 +114,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
   int out_of_date_ok = 0;
   t_uchar * file_list_file = 0;
   int opt_end_with_double_dash = 0;
+  int escape_classes = arch_escape_classes;
 
   dir = str_save (0, ".");
 
@@ -214,6 +217,12 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
             file_list_file = str_save (0, option->arg_string);
             break;
           }
+
+	case opt_unescaped:
+	  {
+	    escape_classes = 0;
+	    break;
+	  }
         }
     }
 
@@ -300,6 +309,12 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
     archive = arch_parse_package_name (arch_ret_archive, default_archive, vsnspec);
     version = arch_parse_package_name (arch_ret_non_archive, 0, vsnspec);
 
+    if (arch_is_system_package_name (version))
+      {
+        safe_printfmt (2, "%s: user's can not commit to system versions\n  version: %s\n", argv[0], version);
+        exit (2);
+      }
+
     if (log_text || summary)
       {
         if (! summary)
@@ -352,7 +367,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
       int status;
 
       lint = arch_tree_lint (tree_root);
-      status = arch_print_tree_lint_report (2, lint);
+      status = arch_print_tree_lint_report (2, lint, escape_classes);
 
       if ((status < 0) || (strict && status))
         {
@@ -392,7 +407,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
             }
           else if (fix)
             {
-              safe_printfmt (2, "%s: can not --fix before --seal", argv[0]);
+              safe_printfmt (2, "%s: can not --fix before --seal\n", argv[0]);
               exit (2);
             }
           else
@@ -412,7 +427,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
             }
           else if (fix)
             {
-              safe_printfmt (2, "%s: can not --fix before --seal", argv[0]);
+              safe_printfmt (2, "%s: can not --fix before --seal\n", argv[0]);
               exit (2);
             }
           else
@@ -427,7 +442,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
         {
           if (seal)
             {
-              safe_printfmt (2, "%s: version already sealed", argv[0]);
+              safe_printfmt (2, "%s: version already sealed\n", argv[0]);
               exit (2);
             }
           else if (fix)
@@ -437,7 +452,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
             }
           else
             {
-              safe_printfmt (2, "%s: cannot commit to sealed version with --fix",
+              safe_printfmt (2, "%s: cannot commit to sealed version without --fix\n",
                              argv[0]);
               exit (2);
             }
@@ -448,7 +463,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
         {
           if (seal)
             {
-              safe_printfmt (2, "%s: version already sealed", argv[0]);
+              safe_printfmt (2, "%s: version already sealed\n", argv[0]);
               exit (2);
             }
           else if (fix)
@@ -458,7 +473,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
             }
           else
             {
-              safe_printfmt (2, "%s: cannot commit to sealed version with --fix",
+              safe_printfmt (2, "%s: cannot commit to sealed version without --fix\n",
                              argv[0]);
               exit (2);
             }
@@ -488,7 +503,7 @@ arch_cmd_commit (t_uchar * program_name, int argc, char * argv[])
     desired_level = arch_form_patch_level (desired_level_type, desired_n);
     revision = str_alloc_cat_many (0, version, "--", desired_level, str_end);
 
-    arch_commit (1, arch, revision, tree_root, log, file_list);
+    arch_commit (1, arch, revision, tree_root, log, file_list, escape_classes);
 
     safe_printfmt (1, "* committed %s/%s\n", archive, revision);
 
@@ -540,7 +555,7 @@ arch_make_edit_log (t_uchar *tree_root, t_uchar *archive, t_uchar *version)
       safe_printfmt (2, "arch_make_edit_log: editor exited with error code %d\n", ecode);
       exit (2);
     }
-  
+
   new_text = file_contents (log_path);
   if (!str_cmp (text, new_text))
     {
