@@ -19,6 +19,7 @@
 #include "tla/libarch/archive-setup.h"
 #include "tla/libarch/import.h"
 #include "tla/libarch/cmd.h"
+#include "tla/libarch/cmdutils.h"
 #include "tla/libarch/cmd-import.h"
 
 
@@ -56,7 +57,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
   OP (opt_log_msg, "L", "log-message TEXT", 1, \
       "log with TEXT") \
   OP (opt_setup, "S", "setup", 0, \
-      "Use `archive-setup' if necessary.")
+      "Use `archive-setup' if necessary.") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 t_uchar arch_cmd_import_help[] = ("archive a full-source base-0 revision\n"
                                   "Archive a from-scratch base revision of the project tree\n"
@@ -90,6 +93,7 @@ arch_cmd_import (t_uchar * program_name, int argc, char * argv[])
   t_uchar * log_text = 0;
   t_uchar * summary = 0;
   int setup = 0;
+  int escape_classes = arch_escape_classes;
 
   dir = str_save (0, ".");
 
@@ -157,6 +161,12 @@ arch_cmd_import (t_uchar * program_name, int argc, char * argv[])
             setup = 1;
             break;
           }
+
+	case opt_unescaped:
+	  {
+	    escape_classes = 0;
+	    break;
+	  }
         }
     }
 
@@ -213,6 +223,12 @@ arch_cmd_import (t_uchar * program_name, int argc, char * argv[])
     archive = arch_parse_package_name (arch_ret_archive, default_archive, vsnspec);
     version = arch_parse_package_name (arch_ret_non_archive, 0, vsnspec);
 
+    if (arch_is_system_package_name (version))
+      {
+        safe_printfmt (2, "%s: user's can not import into system versions\n  version: %s\n", argv[0], version);
+        exit (2);
+      }
+
     if (log_text || summary)
       {
         if (! summary)
@@ -265,7 +281,7 @@ arch_cmd_import (t_uchar * program_name, int argc, char * argv[])
 
       if (log_entries)
         {
-          safe_printfmt (2, "%s: tree already has patch log entries for version\n    tree: %s\n    version: %s/%s",
+          safe_printfmt (2, "%s: tree already has patch log entries for version\n    tree: %s\n    version: %s/%s\n",
                          argv[0], tree_root, archive, version);
           exit (1);
         }
@@ -277,7 +293,7 @@ arch_cmd_import (t_uchar * program_name, int argc, char * argv[])
       int status;
 
       lint = arch_tree_lint (tree_root);
-      status = arch_print_tree_lint_report (2, lint);
+      status = arch_print_tree_lint_report (2, lint, escape_classes);
 
       if (status < 0)
         {
@@ -290,6 +306,7 @@ arch_cmd_import (t_uchar * program_name, int argc, char * argv[])
       arch_setup_archive_simple (1, archive, version);
 
     arch = arch_archive_connect (archive, 0);
+    arch_check_for (arch, arch_req_version, version);
     arch_import (arch, version, tree_root,  log);
 
     safe_printfmt (1, "* imported %s/%s\n", archive, version);
