@@ -58,19 +58,17 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "Operate on project tree in DIR (default `.')") \
   OP (opt_dest, 0, "dest DEST", 1, \
       "Instead of modifying the project tree in-place,\n" \
-      "make a copy of it to DEST and apply the result to that")
+      "make a copy of it to DEST and apply the result to that") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 
-t_uchar arch_cmd_join_branch_help[] = ("construct a project tree for a version\n"
+t_uchar arch_cmd_join_branch_help[] = ("add a version as an ancestor of a project tree\n"
 
-                                       "REVISION must be a continuation (e.g. tag) of an ancestor of the\n"
-                                       "project tree.  The command replays the changeset for REVISION in the\n"
-                                       "project tree, which has the effect of adding the log for the branch\n"
-                                       "(making the tag an ancestor of the resulting tree).\n"
-                                       "\n"
-                                       "As an exception to the usual rules, REVISION may be abbreviated\n"
-                                       "as a version name, in which case the base-0 revision of that\n"
-                                       "version is indicated.\n");
+                                       "VERSION--base-0 must be a continuation (e.g. tag). The command replays the\n"
+                                       "changeset for VERSION--base-0 in the project tree, which has the effect of\n"
+                                       "adding the log for the branch (making the tag an ancestor of the resulting\n"
+                                       "tree).\n");
 
 enum options
 {
@@ -93,6 +91,7 @@ arch_cmd_join_branch (t_uchar * program_name, int argc, char * argv[])
   t_uchar * default_archive = 0;
   t_uchar * upon = 0;
   t_uchar * dest = 0;
+  int escape_classes = arch_escape_classes;
 
   safe_buffer_fd (1, 0, O_WRONLY, 0);
 
@@ -137,6 +136,12 @@ arch_cmd_join_branch (t_uchar * program_name, int argc, char * argv[])
             dest = str_save (0, option->arg_string);
             break;
           }
+
+	case opt_unescaped:
+	  {
+	    escape_classes = 0;
+	    break;
+	  }
         }
     }
 
@@ -216,7 +221,7 @@ arch_cmd_join_branch (t_uchar * program_name, int argc, char * argv[])
     else
       dest = str_save (0, upon);
 
-    arch_call_cmd (arch_cmd_replay, argv[0], "--dir", dest, fqbase0, 0);
+    arch_call_cmd (arch_cmd_replay, argv[0], "--dir", dest, fqbase0, ((escape_classes == 0) ? "--unescaped" : 0), 0);
 
 
     lim_free (0, archive);
