@@ -32,14 +32,14 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
                                    cfg__std__release_id_string
                                    "\n");
 
-#define OPTS(OP, OP2) \
+#define OPTS(OP) \
   OP (opt_help_msg, "h", "help", 0, \
       "Display a help message and exit.") \
   OP (opt_long_help, "H", 0, 0, \
       "Display a verbose help message and exit.") \
   OP (opt_version, "V", "version", 0, \
-      "Display a release identifier string") \
-  OP2 (opt_version, 0, 0, 0, "and exit.") \
+      "Display a release identifier string\n" \
+      "and exit.") \
   OP (opt_archive, "A", "archive", 1, \
       "Override `my-default-archive'")
 
@@ -49,12 +49,12 @@ t_uchar arch_cmd_library_categories_help[] = ("list the categories in your revis
 
 enum options
 {
-  OPTS (OPT_ENUM, OPT_IGN)
+  OPTS (OPT_ENUM)
 };
 
 static struct opt_desc opts[] =
 {
-  OPTS (OPT_DESC, OPT_DESC)
+  OPTS (OPT_DESC)
     {-1, 0, 0, 0, 0}
 };
 
