@@ -40,7 +40,7 @@ static t_uchar * rbrowse_revision_range (t_uchar *latest_revision);
 
 
 
-static t_uchar * usage = "[options] [LIMIT REGEX]";
+static t_uchar * usage = "[options] [[ARCHIVE_NAME/]LIMIT_REGEX]";
 static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
                                    "\n"
                                    "Copyright 2003 Tom Lord\n"
@@ -229,28 +229,48 @@ arch_cmd_rbrowse (t_uchar * program_name, int argc, char * argv[])
   if (argc > 2)
     goto usage_error;
 
-  if (argc == 2)
-    limit_regex = str_save(0, argv[1]);
+  /* This is slightly long. Basically, we're checking to see if we were
+   * given an limit regex that looks like archive_name/regex. If so, then
+   * we need to split them up. */
+  if (argc == 1)
+    limit_regex = str_save (0, ".*");
   else
-    limit_regex = str_save(0, ".*");
+    {
+      t_uchar *slash;
+
+      /* Did the user give us something resembling archive/regex ? */
+      if ( 0 == (slash = str_chr_index (argv[1], '/')))
+        limit_regex = str_save (0, argv[1]);
+      else
+        {
+
+          if (default_archive)
+            panic ("Archive specified twice! ");
+
+          limit_regex = str_save(0, argv[1]);
+          default_archive = str_separate (&limit_regex, "/");
+
+          if (! limit_regex)
+            limit_regex = str_save (0, ".*");
+            /* limit_regex unset, so make something sensible */
+        }
+    }
+
+  if (!default_archive)
+    default_archive = arch_my_default_archive (default_archive);
 
   if (regcomp(&limit_regex_needle, limit_regex, REG_EXTENDED))
     panic ("Invalid regular expression given for limit.");
 
-
   if (! patch_regex)
     patch_regex = str_save(0, ".*");
 
   if (regcomp(&patch_regex_needle, patch_regex, REG_EXTENDED))
     panic ("Invalid regular expression given for patch.");
 
-
   if (since_filename)
     saved_since = rbrowse_load_since (since_filename);
 
-  if (!default_archive)
-    default_archive = arch_my_default_archive (default_archive);
-
   if (!arch_valid_archive_name (default_archive))
     panic ("Invalid archive given");
 
