@@ -18,6 +18,7 @@
 #include "tla/libarch/archive.h"
 #include "tla/libarch/archives.h"
 #include "tla/libarch/pfs.h"
+#include "tla/libarch/cmdutils.h"
 #include "tla/libarch/cmd-register-archive.h"
 #include "tla/libawk/trim.h"
 
@@ -47,6 +48,8 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
       "and exit.") \
   OP (opt_force, "f", "force", 0, \
       "overwrite existing location") \
+  OP (opt_present_ok, 0, "present-ok", 0, \
+      "return 0 even if archive exists") \
   OP (opt_delete, "d", "delete", 0, \
       "delete archive registration")
 
@@ -92,9 +95,11 @@ arch_cmd_register_archive (t_uchar * program_name, int argc, char * argv[])
   struct opt_parsed * option;
   int delete;
   int force;
+  int quietly_fail;
 
   delete = 0;
   force = 0;
+  quietly_fail = 0;
 
   safe_buffer_fd (1, 0, O_WRONLY, 0);
 
@@ -124,12 +129,15 @@ arch_cmd_register_archive (t_uchar * program_name, int argc, char * argv[])
             delete = 1;
             break;
           }
-
         case opt_force:
           {
             force = 1;
             break;
           }
+        case opt_present_ok:
+          {
+            quietly_fail = ARCH_REG_FAIL_NOFAIL;
+          }
         }
     }
 
@@ -160,18 +168,20 @@ arch_cmd_register_archive (t_uchar * program_name, int argc, char * argv[])
         {
           archive_name = argv[1];
           archive_location = argv[2];
+          arch_check_uri (archive_location);
         }
       else
         {
           struct arch_archive * archive = 0;
 
           archive_location = argv[1];
+          arch_check_uri (archive_location);
 
           archive = arch_archive_connect_location (0, archive_location, 0);
           archive_name = trim_surrounding_ws (arch_get_meta_info (archive, "name"));
           arch_archive_close (archive);
-
-      safe_printfmt (1, "Registering archive: %s\n", archive_name);
+          
+          safe_printfmt (1, "Registering archive: %s\n", archive_name);
 
         }
 
@@ -182,7 +192,7 @@ arch_cmd_register_archive (t_uchar * program_name, int argc, char * argv[])
           exit (1);
         }
 
-      arch_set_archive_location (archive_name, archive_location, force, 0);
+      arch_set_archive_location (archive_name, archive_location, force, quietly_fail);
     }
 
   return 0;
