@@ -14,6 +14,8 @@
 #include "tla/libarch/namespace.h"
 #include "tla/libarch/project-tree.h"
 #include "tla/libarch/star-merge.h"
+#include "tla/libarch/cmdutils.h"
+#include "tla/libarch/cmd.h"
 #include "tla/libarch/cmd-star-merge.h"
 
 
@@ -51,7 +53,9 @@ static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
   OP (opt_three_way, "t", "three-way", 0, \
       "Perform a 3-way (diff3-style) merge.") \
   OP (opt_dir, "d", "dir DIR", 1, \
-      "Operate on project tree in DIR (default `.')")
+      "Operate on project tree in DIR (default `.')") \
+  OP (opt_unescaped, 0, "unescaped", 0, \
+      "show filenames in unescaped form")
 
 
 t_uchar arch_cmd_star_merge_help[] = ("merge mutually merged branches\n"
@@ -127,6 +131,7 @@ arch_cmd_star_merge (t_uchar * program_name, int argc, char * argv[])
   t_uchar * dir = 0;
   int three_way = 0;
   int forward = 0;
+  int escape_classes = arch_escape_classes;
 
   default_archive = 0;
 
@@ -192,6 +197,12 @@ arch_cmd_star_merge (t_uchar * program_name, int argc, char * argv[])
             dir = str_save (0, option->arg_string);
             break;
           }
+        
+        case opt_unescaped:
+          {
+            escape_classes = 0;
+            break;
+          }
         }
     }
 
@@ -254,12 +265,18 @@ arch_cmd_star_merge (t_uchar * program_name, int argc, char * argv[])
 
     if (arch_valid_package_name (from_spec, arch_maybe_archive, arch_req_patch_level, 0))
       {
+        if (argc == 2)
+          arch_check_for (from_arch, arch_req_patch_level, from_spec);
+        
         from_revision = arch_parse_package_name (arch_ret_non_archive, 0, from_spec);
       }
     else
       {
         t_uchar * from_version = 0;
         rel_table revisions = 0;
+        
+        if (argc == 2) 
+	  arch_check_for (from_arch, arch_req_version, from_spec);
 
         from_version = arch_parse_package_name (arch_ret_non_archive, 0, from_spec);
 
@@ -280,7 +297,7 @@ arch_cmd_star_merge (t_uchar * program_name, int argc, char * argv[])
 
     cache_dir = file_name_directory_file (0, tree_root);
 
-    exit_status = arch_star_merge (1, from_arch, from_archive, from_revision, tree_root, 0, to_archive, to_version, cache_dir, changeset, three_way, forward);
+    exit_status = arch_star_merge (1, from_arch, from_archive, from_revision, tree_root, 0, to_archive, to_version, cache_dir, changeset, three_way, forward, escape_classes);
 
     lim_free (0, from_spec);
     lim_free (0, from_archive);
