@@ -15,6 +15,7 @@
 #include "tla/libarch/my.h"
 #include "tla/libarch/archive.h"
 #include "tla/libarch/cmd-versions.h"
+#include "tla/libarch/cmdutils.h"
 
 
 
@@ -144,6 +145,8 @@ arch_cmd_versions (t_uchar * program_name, int argc, char * argv[])
     branch = arch_parse_package_name (arch_ret_package, 0, branch_spec);
 
     arch = arch_archive_connect (archive, 0);
+    if (argc == 2)
+      arch_check_for (arch, arch_req_package, branch);
     versions = arch_archive_versions (arch, branch);
 
     if (reverse)
