@@ -1,7 +1,7 @@
 /* cmds.c:
  *
  ****************************************************************
- * Copyright (C) 2003 Tom Lord
+ * Copyright (C) 2003, 2004 Tom Lord
  *
  * See the file "COPYING" for further information about
  * the copyright and warranty status of this work.
@@ -36,7 +36,7 @@
 #include "tla/libarch/cmd-rm.h"
 #include "tla/libarch/cmd-apply-delta.h"
 #include "tla/libarch/cmd-apply-changeset.h"
-#include "tla/libarch/cmd-file-diffs.h"
+#include "tla/libarch/cmd-file-diff.h"
 #include "tla/libarch/cmd-file-find.h"
 #include "tla/libarch/cmd-find-pristine.h"
 #include "tla/libarch/cmd-get-changeset.h"
@@ -104,6 +104,7 @@
 #include "tla/libarch/cmd-changes.h"
 #include "tla/libarch/cmd-missing.h"
 #include "tla/libarch/cmd-whereis-archive.h"
+#include "tla/libarch/cmd-escape.h"
 #include "tla/libarch/cmds.h"
 
 
@@ -141,7 +142,8 @@ struct arch_command arch_commands[] =
   {"", 0},
   {"changes", arch_cmd_changes, arch_cmd_changes_help, 0, 0},
   {"what-changed", arch_cmd_changes, arch_cmd_changes_help, "changes", 1},
-  {"file-diffs", arch_cmd_file_diffs, arch_cmd_file_diffs_help, 0, 0},
+  {"file-diff", arch_cmd_file_diff, arch_cmd_file_diff_help, 0, 0},
+  {"file-diffs", arch_cmd_file_diff, arch_cmd_file_diff_help, "file-diff", 1},
 
 
   {"Project Tree Inventory Commands", 0, 0, 0, 0},
@@ -277,7 +279,7 @@ struct arch_command arch_commands[] =
 
   {"changes", arch_cmd_changes, arch_cmd_changes_help, 0, 0},
   {"what-changed", arch_cmd_changes, arch_cmd_changes_help, "changes", 1},
-  {"file-diffs", arch_cmd_file_diffs, arch_cmd_file_diffs_help, 0, 0},
+  {"file-diff", arch_cmd_file_diff, arch_cmd_file_diff_help, 0, 0},
   {"file-find", arch_cmd_file_find, arch_cmd_file_find_help, 0, 0},
   {"", 0},
   {"pristines", arch_cmd_pristines, arch_cmd_pristines_help, 0, 0},
@@ -313,6 +315,7 @@ struct arch_command arch_commands[] =
 
   {"parse-package-name", arch_cmd_parse_package_name, arch_cmd_parse_package_name_help, 0, 0},
   {"valid-package-name", arch_cmd_valid_package_name, arch_cmd_valid_package_name_help, 0, 0},
+  {"escape", arch_cmd_escape, arch_cmd_escape_help, 0, 0},
 
 
   {0, },
