@@ -57,11 +57,19 @@ arch_import (struct arch_archive * arch,
   t_uchar * cooked_log = 0;
   t_uchar * my_uid = 0;
   t_uchar * txn_id = 0;
+  t_uchar * revision;
+  
+  revision = str_alloc_cat (0, version, "--base-0");
 
   pristine_path = arch_prepare_import_pristine (&changelog_loc, &cooked_log, tree_root, raw_log, arch->name, version);
 
   my_uid = arch_my_id_uid ();
   txn_id = arch_generate_txn_id ();
+  if (arch_revision_exists (arch, revision))
+    {
+      safe_printfmt (2, "arch_import: the revision already exists\n    revision: %s/%s\n", arch->official_name, revision);
+      exit (2);
+    }
 
   if (arch_archive_lock_revision (&errstr, arch, version, 0, my_uid, txn_id, "base-0"))
     {
@@ -103,16 +111,9 @@ arch_import (struct arch_archive * arch,
 
   arch_finish_import (tree_root, arch->name, version, pristine_path, changelog_loc);
 
-  {
-    t_uchar * revision = 0;
-
-    revision = str_alloc_cat (0, version, "--base-0");
-
-    arch_run_hook ("import", "ARCH_ARCHIVE", arch->name, "ARCH_REVISION", revision, "ARCH_TREE_ROOT", tree_root, 0);
-
-    lim_free (0, revision);
-  }
-
+  arch_run_hook ("import", "ARCH_ARCHIVE", arch->name, "ARCH_REVISION", revision, "ARCH_TREE_ROOT", tree_root, 0);
+  
+  lim_free (0, revision);
   lim_free (0, changelog_loc);
   lim_free (0, pristine_path);
   lim_free (0, cooked_log);
