@@ -11,16 +11,19 @@
 #include "hackerlab/bugs/panic.h"
 #include "hackerlab/mem/mem.h"
 #include "hackerlab/char/str.h"
+#include "hackerlab/char/char-class.h"
 #include "hackerlab/arrays/ar.h"
 #include "hackerlab/sort/qsort.h"
 #include "hackerlab/fs/file-names.h"
 #include "hackerlab/fs/cwd.h"
 #include "hackerlab/vu/safe.h"
+#include "hackerlab/char/pika-escaping-utils.h"
 #include "tla/libfsutils/ensure-dir.h"
 #include "tla/libfsutils/dir-as-cwd.h"
 #include "tla/libfsutils/copy-file.h"
 #include "tla/libfsutils/link-target.h"
 #include "tla/libawk/associative.h"
+#include "tla/libarch/arch.h"
 #include "tla/libarch/diffs.h"
 #include "tla/libarch/invent.h"
 #include "tla/libarch/project-tree.h"
@@ -44,7 +47,8 @@ static void emit_dir_patches (rel_table * orig_index_names_output,
                               t_uchar * orig,
                               t_uchar * orig_loc,
                               t_uchar * mod,
-                              t_uchar * mod_loc);
+                              t_uchar * mod_loc,
+                              int escape_classes);
 static void emit_file_or_symlink_patches (rel_table * orig_index_names_output,
                                           rel_table * mod_index_names_output,
                                           struct arch_make_changeset_report * report,
@@ -55,7 +59,8 @@ static void emit_file_or_symlink_patches (rel_table * orig_index_names_output,
                                           t_uchar * mod_loc,
                                           t_uchar * id,
                                           assoc_table inode_sig_shortcuts_of_mod,
-					  int link_same);
+                                          int link_same,
+                                          int escape_classes);
 static void compute_parent_dir_closure (rel_table * orig_newname_dirs, rel_table * mod_newname_dirs,
                                         rel_table orig_newname_files, rel_table mod_newname_files,
                                         assoc_table orig_dir_loc_of, assoc_table orig_dir_id_of,
@@ -73,7 +78,6 @@ static void container_closure (rel_table * locs_out,
                                t_uchar * deep_id,
                                assoc_table dir_loc_of,
                                assoc_table container_dir_id_of_dir_id);
-
 static void link_force (t_uchar *old_path, t_uchar *new_path);
 
 
@@ -142,9 +146,9 @@ arch_make_changeset (struct arch_make_changeset_report * report,
                      enum arch_inventory_category untagged_source_category,
                      rel_table limits_spec,
                      assoc_table inode_sig_shortcuts_of_mod,
-		     int link_same)
+		     int link_same,
+                     int escape_classes)
 {
-  int ign;
   int here_fd;
   t_uchar * orig;
   t_uchar * mod;
@@ -289,9 +293,7 @@ arch_make_changeset (struct arch_make_changeset_report * report,
   safe_mkdir (dest_spec, 0777);
   safe_chdir (dest_spec);
 
-  dest = current_working_directory (&ign, 0);
-  if (!dest)
-    panic ("can't compute current working directory (arch_make_changeset)");
+  dest = safe_current_working_directory ();
 
   safe_fchdir (here_fd);
 
@@ -300,8 +302,10 @@ arch_make_changeset (struct arch_make_changeset_report * report,
   mem_set0 ((t_uchar *)&report->orig_index, sizeof (report->orig_index));
   mem_set0 ((t_uchar *)&report->mod_index, sizeof (report->mod_index));
 
-  arch_changeset_inventory (&report->orig_index, orig_root, orig, method, untagged_source_category);
-  arch_changeset_inventory (&report->mod_index, mod_root, mod, method, untagged_source_category);
+  arch_changeset_inventory (&report->orig_index, orig_root, orig, method,
+                            untagged_source_category, escape_classes);
+  arch_changeset_inventory (&report->mod_index, mod_root, mod, method,
+                            untagged_source_category, escape_classes);
 
   orig_full_index = rel_copy_table (report->orig_index.dirs);
   rel_append_x (&orig_full_index, report->orig_index.files);
@@ -334,19 +338,19 @@ arch_make_changeset (struct arch_make_changeset_report * report,
        * that some item hasn't been modified, it's left out of this translation.
        */
 
-     /* Three cases of items to deal with in the fake inventory:
-      *
-      * 1) Items common to both ORIG and MOD
-      * 2) Items unique to ORIG
-      * 3) Items unique to MOD
-      *
-      *
-      * Little lemma:  for each (canonicalized) limit $PREFIX/$TAIL,
-      * the item in MOD with loc $PREFIX must exist in ORIG.   Because,
-      * otherwise, the limit wants to add files to dirs that have not been
-      * added to ORIG (a user error).
-      *
-      */
+      /* Three cases of items to deal with in the fake inventory:
+       *
+       * 1) Items common to both ORIG and MOD
+       * 2) Items unique to ORIG
+       * 3) Items unique to MOD
+       *
+       *
+       * Little lemma:  for each (canonicalized) limit $PREFIX/$TAIL,
+       * the item in MOD with loc $PREFIX must exist in ORIG.   Because,
+       * otherwise, the limit wants to add files to dirs that have not been
+       * added to ORIG (a user error).
+       *
+       */
 
       /* currently, limits is just [0] == mod_path */
 
@@ -532,12 +536,20 @@ arch_make_changeset (struct arch_make_changeset_report * report,
           lim = rel_n_records (report->removed_dirs);
           for (x = 0; x < lim; ++x)
             {
-              invoke_report_callback (report, "D/ %s\n", no_dot (report->removed_dirs[x][0]));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       report->removed_dirs[x][0]);
+              invoke_report_callback (report, "D/ %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
           lim = rel_n_records (report->added_dirs);
           for (x = 0; x < lim; ++x)
             {
-              invoke_report_callback (report, "A/ %s\n", no_dot (report->added_dirs[x][0]));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       report->added_dirs[x][0]);
+              invoke_report_callback (report, "A/ %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
         }
     }
@@ -564,12 +576,20 @@ arch_make_changeset (struct arch_make_changeset_report * report,
           lim = rel_n_records (report->removed_files);
           for (x = 0; x < lim; ++x)
             {
-              invoke_report_callback (report, "D  %s\n", no_dot (report->removed_files[x][0]));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       report->removed_files[x][0]);
+              invoke_report_callback (report, "D  %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
           lim = rel_n_records (report->added_files);
           for (x = 0; x < lim; ++x)
             {
-              invoke_report_callback (report, "A  %s\n", no_dot (report->added_files[x][0]));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       report->added_files[x][0]);
+              invoke_report_callback (report, "A  %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
         }
     }
@@ -587,13 +607,13 @@ arch_make_changeset (struct arch_make_changeset_report * report,
       emit_file_or_symlink_patches (&orig_files_index_names_output, &mod_files_index_names_output, report,
                                     patches_dir, orig, files_in_both[x][0], mod, files_in_both[x][1],
                                     files_in_both[x][2],
-                                    inode_sig_shortcuts_of_mod, link_same);
+                                    inode_sig_shortcuts_of_mod, link_same, escape_classes);
     }
   lim = rel_n_records (dirs_in_both);
   for (x = 0; x < lim; ++x)
     {
       emit_dir_patches (&orig_dirs_index_names_output, &mod_dirs_index_names_output, report,
-                        patches_dir, orig, dirs_in_both[x][0], mod, dirs_in_both[x][1]);
+                        patches_dir, orig, dirs_in_both[x][0], mod, dirs_in_both[x][1], escape_classes);
     }
 
   /****************************************************************
@@ -668,10 +688,10 @@ arch_make_changeset (struct arch_make_changeset_report * report,
     mod_dirs_index_fd = safe_open (mod_dirs_index_file, O_WRONLY | O_CREAT | O_EXCL, 0666);
     mod_files_index_fd = safe_open (mod_files_index_file, O_WRONLY | O_CREAT | O_EXCL, 0666);
 
-    rel_print_table_sp (orig_dirs_index_fd, orig_dirs_index_final);
-    rel_print_table_sp (orig_files_index_fd, orig_files_index_final);
-    rel_print_table_sp (mod_dirs_index_fd, mod_dirs_index_final);
-    rel_print_table_sp (mod_files_index_fd, mod_files_index_final);
+    rel_print_pika_escape_iso8859_1_table_sp (orig_dirs_index_fd, arch_escape_classes, orig_dirs_index_final);
+    rel_print_pika_escape_iso8859_1_table_sp (orig_files_index_fd, arch_escape_classes, orig_files_index_final);
+    rel_print_pika_escape_iso8859_1_table_sp (mod_dirs_index_fd, arch_escape_classes, mod_dirs_index_final);
+    rel_print_pika_escape_iso8859_1_table_sp (mod_files_index_fd, arch_escape_classes, mod_files_index_final);
 
     safe_close (orig_dirs_index_fd);
     safe_close (orig_files_index_fd);
@@ -691,7 +711,15 @@ arch_make_changeset (struct arch_make_changeset_report * report,
             rel_add_records (&report->renamed_dirs, rel_copy_record (renamed_dirs[x]), 0);
             if (report->callback)
               {
-                invoke_report_callback (report, "/> %s\t%s\n", no_dot (renamed_dirs[x][0]), no_dot (renamed_dirs[x][1]));
+                t_uchar * escape_tmp0;
+                t_uchar * escape_tmp1;
+                escape_tmp0 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                          report->renamed_dirs[x][0]);
+                escape_tmp1 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                          report->renamed_dirs[x][1]);
+                invoke_report_callback (report, "/> %s\t%s\n", no_dot (escape_tmp0), no_dot (escape_tmp1));
+                lim_free (0, escape_tmp1);
+                lim_free (0, escape_tmp0);
               }
           }
         lim = rel_n_records (renamed_files);
@@ -700,7 +728,15 @@ arch_make_changeset (struct arch_make_changeset_report * report,
             rel_add_records (&report->renamed_files, rel_copy_record (renamed_files[x]), 0);
             if (report->callback)
               {
-                invoke_report_callback (report, "=> %s\t%s\n", no_dot (renamed_files[x][0]), no_dot (renamed_files[x][1]));
+                t_uchar * escape_tmp0;
+                t_uchar * escape_tmp1;
+                escape_tmp0 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                          report->renamed_files[x][0]);
+                escape_tmp1 = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                          report->renamed_files[x][1]);
+                invoke_report_callback (report, "=> %s\t%s\n", no_dot (escape_tmp0), no_dot (escape_tmp1));
+                lim_free (0, escape_tmp1);
+                lim_free (0, escape_tmp0);
               }
           }
       }
@@ -853,10 +889,10 @@ arch_changeset_rewrite_indexes (t_uchar * changeset_path, struct arch_changeset_
   rel_sort_table_by_field (0, report->mod_dirs_index, 1);
   rel_sort_table_by_field (0, report->mod_files_index, 1);
 
-  rel_print_table_sp (orig_dirs_index_fd, report->orig_dirs_index);
-  rel_print_table_sp (orig_files_index_fd, report->orig_files_index);
-  rel_print_table_sp (mod_dirs_index_fd, report->mod_dirs_index);
-  rel_print_table_sp (mod_files_index_fd, report->mod_files_index);
+  rel_print_pika_escape_iso8859_1_table_sp (orig_dirs_index_fd, arch_escape_classes, report->orig_dirs_index);
+  rel_print_pika_escape_iso8859_1_table_sp (orig_files_index_fd, arch_escape_classes, report->orig_files_index);
+  rel_print_pika_escape_iso8859_1_table_sp (mod_dirs_index_fd, arch_escape_classes, report->mod_dirs_index);
+  rel_print_pika_escape_iso8859_1_table_sp (mod_files_index_fd, arch_escape_classes, report->mod_files_index);
 
   safe_close (orig_dirs_index_fd);
   safe_close (orig_files_index_fd);
@@ -1112,15 +1148,19 @@ write_dir_metadata (t_uchar * file, t_uchar * tree_root, rel_table dirs)
   for (x = 0; x < lim; ++x)
     {
       t_uchar * path;
+      t_uchar * escape_tmp;
       struct stat stat_buf;
 
       path = file_name_in_vicinity (0, tree_root, dirs[x][0]);
       safe_stat (path, &stat_buf);
 
+      escape_tmp = pika_save_escape_iso8859_1 (0, 0, arch_escape_classes, dirs[x][0]);
+
       safe_printfmt (out_fd, "--permissions %lo\t%s\n",
                      (unsigned long)(stat_buf.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO)),
-                     dirs[x][0]);
+                     escape_tmp);
 
+      lim_free (0, escape_tmp);
       lim_free (0, path);
     }
 
@@ -1166,7 +1206,8 @@ emit_dir_patches (rel_table * orig_index_names_output,
                   t_uchar * orig,
                   t_uchar * orig_loc,
                   t_uchar * mod,
-                  t_uchar * mod_loc)
+                  t_uchar * mod_loc,
+                  int escape_classes)
 {
   t_uchar * orig_path;
   t_uchar * mod_path;
@@ -1200,7 +1241,11 @@ emit_dir_patches (rel_table * orig_index_names_output,
           rel_add_records (&report->perms_changed_dirs, rel_make_record (orig_loc, mod_loc, 0), 0);
           if (report->callback)
             {
-              invoke_report_callback (report, "-/ %s\n", no_dot (mod_loc));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       mod_loc);
+              invoke_report_callback (report, "-/ %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
         }
 
@@ -1241,7 +1286,8 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
                               t_uchar * mod_loc,
                               t_uchar * id,
                               assoc_table inode_sig_shortcuts_of_mod,
-			      int link_same)
+			      int link_same,
+                              int escape_classes)
 {
   t_uchar * orig_path;
   t_uchar * mod_path;
@@ -1284,7 +1330,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
               rel_add_records (&report->modified_files, rel_make_record (orig_loc, mod_loc, 0), 0);
               if (report->callback)
                 {
-                  invoke_report_callback (report, "-> %s\n", no_dot (mod_loc));
+                  t_uchar * escape_tmp;
+                  escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                           mod_loc);
+                  invoke_report_callback (report, "-> %s\n", no_dot (escape_tmp));
+                  lim_free (0, escape_tmp);
                 }
             }
 
@@ -1327,7 +1377,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
           rel_add_records (&report->modified_files, rel_make_record (orig_loc, mod_loc, 0), 0);
           if (report->callback)
             {
-              invoke_report_callback (report, "lf %s\n", no_dot (mod_loc));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       mod_loc);
+              invoke_report_callback (report, "lf %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
         }
 
@@ -1370,7 +1424,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
           rel_add_records (&report->modified_files, rel_make_record (orig_loc, mod_loc, 0), 0);
           if (report->callback)
             {
-              invoke_report_callback (report, "fl %s\n", no_dot (mod_loc));
+              t_uchar * escape_tmp;
+              escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                       mod_loc);
+              invoke_report_callback (report, "fl %s\n", no_dot (escape_tmp));
+              lim_free (0, escape_tmp);
             }
         }
 
@@ -1410,13 +1468,14 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
 
       is_changed = 0;
       if (arch_file_stats_differ (&orig_stat, &mod_stat) == 0) 
-	 goto path_free;
+        goto path_free;
       if (arch_inode_sigs_differ (&mod_stat, id, inode_sig_shortcuts_of_mod) == 0)
-	{
-	  if (orig_stat.st_dev == mod_stat.st_dev && link_same)
-	    link_force (orig_path, mod_path);
-	  goto path_free;
-	}
+        {
+          if (orig_stat.st_dev == mod_stat.st_dev && link_same)
+            link_force (orig_path, mod_path);
+          goto path_free;
+        }
+
       patch_base_file = file_name_in_vicinity (0, patches, mod_loc);
       patch_file_dir = file_name_directory_file (0, patch_base_file);
 
@@ -1438,7 +1497,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
               rel_add_records (&report->perms_changed_files, rel_make_record (orig_loc, mod_loc, 0), 0);
               if (report->callback)
                 {
-                  invoke_report_callback (report, "-- %s\n", no_dot (mod_loc));
+                  t_uchar * escape_tmp;
+                  escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                           mod_loc);
+                  invoke_report_callback (report, "-- %s\n", no_dot (escape_tmp));
+                  lim_free (0, escape_tmp);
                 }
             }
 
@@ -1467,10 +1530,10 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
 	    link_force (orig_path, mod_path);
 	  goto path_free;
 	}
-      
+
       ensure_directory_exists (patch_file_dir);
       diff_file = str_alloc_cat (0, patch_base_file, ".patch");
-      
+
       vu_unlink (&ign, diff_file);
       diff_fd = safe_open (diff_file, O_RDWR | O_EXCL | O_CREAT, 0666);
 
@@ -1493,11 +1556,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
             {
               diff_pseudo_magic[amt] = 0;
               if (!str_casecmp (diff_pseudo_magic, "binary files "))
-                  diff_stat = 2;
+                diff_stat = 2;
             }
         }
       if (diff_stat == 2)
-	  safe_unlink (diff_file);
+        safe_unlink (diff_file);
 
 
       /* diff_stat: 0 == same, 1 == patch in diff_fd, 2 == differing binary files
@@ -1516,7 +1579,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
                 rel_add_records (&report->modified_files, rel_make_record (orig_loc, mod_loc, 0), 0);
                 if (report->callback)
                   {
-                    invoke_report_callback (report, "M  %s\n", no_dot (mod_loc));
+                    t_uchar * escape_tmp;
+                    escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                             mod_loc);
+                    invoke_report_callback (report, "M  %s\n", no_dot (escape_tmp));
+                    lim_free (0, escape_tmp);
                   }
               }
 
@@ -1539,7 +1606,11 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
                 rel_add_records (&report->modified_files, rel_make_record (orig_loc, mod_loc, 0), 0);
                 if (report->callback)
                   {
-                    invoke_report_callback (report, "Mb %s\n", no_dot (mod_loc));
+                    t_uchar * escape_tmp;
+                    escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes,
+                                                             mod_loc);
+                    invoke_report_callback (report, "Mb %s\n", no_dot (escape_tmp));
+                    lim_free (0, escape_tmp);
                   }
               }
 
@@ -1581,7 +1652,7 @@ emit_file_or_symlink_patches (rel_table * orig_index_names_output,
       lim_free (0, patch_file_dir);
     }
 
-  path_free:
+ path_free:
   lim_free (0, orig_path);
   lim_free (0, mod_path);
 }
