@@ -12,6 +12,7 @@
 #include "hackerlab/os/errno.h"
 #include "hackerlab/os/errno-to-string.h"
 #include "hackerlab/char/char-class.h"
+#include "hackerlab/char/pika-escaping-utils.h"
 #include "hackerlab/char/str.h"
 #include "hackerlab/fs/file-names.h"
 #include "hackerlab/vu/safe.h"
@@ -22,6 +23,7 @@
 #include "tla/libfsutils/rmrf.h"
 #include "tla/libfsutils/copy-file.h"
 #include "tla/libawk/trim.h"
+#include "tla/libarch/arch.h"
 #include "tla/libarch/namespace.h"
 #include "tla/libarch/merge-points.h"
 #include "tla/libarch/project-tree.h"
@@ -883,6 +885,7 @@ void
 arch_print_log_list_header (int out_fd, t_uchar * header, rel_table list, int field)
 {
   int num_recs = rel_n_records (list);
+  int escape_classes = arch_escape_classes;
 
   if (num_recs > 0)
     {
@@ -896,12 +899,15 @@ arch_print_log_list_header (int out_fd, t_uchar * header, rel_table list, int fi
 
       for (x = 0; x < num_recs; ++x)
         {
-          char *val = no_dot (list[x][field]);
-          size_t len_this = str_length (val);
+          t_uchar * escape_tmp = 0;
+          size_t len_this;
+
+          escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, no_dot (list[x][field]));
+          len_this = str_length (escape_tmp);
 
           if (!any_on_this_line)
             {
-              safe_printfmt (out_fd, "%s", val);
+              safe_printfmt (out_fd, "%s", escape_tmp);
               col += len_this;
               any_on_this_line = 1;
             }
@@ -909,17 +915,19 @@ arch_print_log_list_header (int out_fd, t_uchar * header, rel_table list, int fi
             {
               if ((col + 1 + len_this) >= 60)
                 {
-                  safe_printfmt (out_fd, "\n    %s", val);
+                  safe_printfmt (out_fd, "\n    %s", escape_tmp);
                   col = 4 + len_this;
                   any_on_this_line = 1;
                 }
               else
                 {
-                  safe_printfmt (out_fd, " %s", val);
+                  safe_printfmt (out_fd, " %s", escape_tmp);
                   col += len_this;
                   any_on_this_line = 1;
                 }
             }
+
+          lim_free (0, escape_tmp);
         }
       safe_printfmt (out_fd, "\n");
     }
@@ -930,6 +938,7 @@ void
 arch_print_log_pairs_header (int out_fd, t_uchar * header, rel_table list, int field_a, int field_b)
 {
   int num_recs = rel_n_records (list);
+  int escape_classes = arch_escape_classes;
 
   if (num_recs > 0)
     {
@@ -942,9 +951,15 @@ arch_print_log_pairs_header (int out_fd, t_uchar * header, rel_table list, int f
 
       for (x = 0; x < num_recs; ++x)
         {
+          t_uchar * escape_tmp_a;
+          t_uchar * escape_tmp_b;
+          escape_tmp_a = pika_save_escape_iso8859_1 (0, 0, escape_classes, list[x][field_a]);
+          escape_tmp_b = pika_save_escape_iso8859_1 (0, 0, escape_classes, list[x][field_b]);
           safe_printfmt (out_fd, "%s%s %s", (once ? "\n    " : ""),
-                         no_dot (list[x][field_a]), no_dot (list[x][field_b]));
+                         no_dot (escape_tmp_a), no_dot (escape_tmp_b));
           once = 1;
+          lim_free (0, escape_tmp_b);
+          lim_free (0, escape_tmp_a);
         }
       safe_printfmt (out_fd, "\n");
     }
