@@ -621,12 +621,14 @@ pfs_ftp_directory_files (struct arch_pfs_session * p, t_uchar * path, int soft_e
 
           if (0 > file_in_fd)
             {
-              /* Workaround for buggy ftp servers that respond a 550 code for empty directories
+              /* Workaround for buggy ftp servers that respond a 450 or 550 code for empty directories.
+               * 450 is reported by proftpd:
+               *    "450 No files found"
                * The pfs_ftp_is_dir ensures the directory exists as it chdir to the given path.
                * Btw, we could not have used pfs_ftp_file_exists otherwise we would have created
                * an infinte function call loop because pfs_ftp_file_exists relies on
                * pfs_ftp_directory_files */
-              if (code != 550 || !pfs_ftp_is_dir (p, path))
+              if ((code != 450 && code != 550) || !pfs_ftp_is_dir (p, path))
                 goto put_error;
             }
           else
