@@ -1,7 +1,7 @@
 /* pfs-sftp.c:
  *
  ****************************************************************
- * Copyright (C) 2002, 2003 Scott Parish
+ * Copyright (C) 2002 - 2004 Scott Parish
  * Copyright (C) 2003 Tom Lord
  *
  * See the file "COPYING" for further information about
@@ -37,6 +37,9 @@
 /* TODO: look through ssh/tcp/ethernet specks to find an optimal size */
 #define SFTP_RWSIZE 16384
 
+/* PipeLine-REQuests: num r/w requests open at once for get() and put() */
+#define MAX_PL_REQS 16 
+
 #undef MIN
 #define MIN(A,B) ((A) < (B) ? (A) : (B))
 #define LISTING_FILE ".listing"
@@ -81,6 +84,7 @@ struct arch_pfs_sftp_session
 
   char * cwd;
   t_uint32 request_id;
+  t_uint32 plreqs;
 
   int in_fd;
   int out_fd;
@@ -317,71 +321,13 @@ arch_pfs_sftp_connect (t_uchar * uri)
   mem_set0 ((t_uchar *)answer, sizeof (*answer));
   answer->pfs.vtable = &sftp_pfs_fns;
 
-  {
-    t_uchar * start;
-    t_uchar * stop;
-    t_uchar * host_part = 0;
-    t_uchar * at;
-    t_uchar * h0;
-    t_uchar * h1;
-    t_uchar * colon;
-
-    /* [user@]hostname[:port]/path
-     * ^    ^ ^       ^      ^
-     * |    | |       |      |
-     * |   at?|       |      stop[*]
-     * |      |       |	     0-byte of host_part
-     * |     h0       |
-     * |              |
-     * |              |
-     * |              |
-     * start[*]     colon?
-     * host_part    h1
-     * 
-     * 
-     * start and stop point to one copy of the string
-     * (the original uri which includes the /path).
-     * 
-     * host_part points to another copy of the string 
-     * (local to this block) which all the other pointers
-     * point into.   
-     * 
-     * at? and colon? are 0 if @ and : are not present.
-     * 
-     * The reason to make host_part is so that str_chr_index 
-     * searches are confined to just the host_part.
-     */
-
-
-    start = uri + sizeof("sftp://") - 1;
-    stop = str_chr_index (start, '/');
-    invariant (!!stop);
-    answer->cwd = str_save (0, stop);
-
-    host_part = str_save_n (0, start, stop - start);
-
-    at = str_chr_index (host_part, '@');
-    if (at)
-      h0 = at + 1;
-    else
-      h0 = host_part;
-
-    colon = str_chr_index (h0, ':');
-    if (colon)
-      h1 = colon;
-    else
-      h1 = h0 + str_length (h0);
-
-    if (at)
-      user = str_save_n (0, host_part, at - host_part);
-
-    if (colon)
-      port = str_save (0, colon + 1);
-
-    hostname = str_save_n (0, h0, h1 - h0);
-
-    lim_free (0, host_part);
-  }
+  if (arch_pfs_sftp_parse_uri (&user, &hostname, &port, &answer->cwd, uri)!=0)
+    {
+      safe_printfmt (2, "Invalid sftp URL\n");
+      /*safe_printfmt (2, "Invalid sftp URL: %s\n", uri); 
+       * can't use URI (may be mangled by arch_pfs_pfs_make_archive, etc */
+      exit (2);
+    }
 
   ssh_argv = 0;
   *(t_uchar **)ar_push ((void **)&ssh_argv, 0, sizeof (t_uchar *)) = cfg__ssh;
@@ -542,7 +488,11 @@ sftp_server_response (struct arch_pfs_sftp_session * p, char **pkt,
 
   if (response != SSH_FXP_VERSION)
     {
-      if (READ_UINT4 (pl + i) != p->request_id)
+      int rq_id_offset = 0; /* request_id offset caused by pipe-lining */
+      if (p->plreqs)
+	rq_id_offset = p->plreqs - 1;
+
+      if (READ_UINT4 (pl + i) != (p->request_id - rq_id_offset))
         {
           if (!soft_errors)
             {
@@ -640,7 +590,7 @@ sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
   int ret, handle_len, i;
   unsigned long long total_read;
   char *handle;
-  int len, errn;
+  int len, errn, eof, numpkts;
 
   file = sftp_abs_path (p->cwd, path);
 
@@ -660,10 +610,10 @@ sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
   handle_len = READ_UINT4 (pkt + i);
   i += 4;
   if (handle_len > pkt_len - i)
-    {
-      if (!soft_errors)
+    { 
+     if (!soft_errors)
         {
-          safe_printfmt(2 , "sftp_client_get: Packet too short(2): did we get garbage from server?\n");
+          safe_printfmt(2, "sftp_client_get: Packet too short(2): did we get garbage from server?\n");
           exit (2);
         }
       else
@@ -676,16 +626,22 @@ sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
   lim_free (0, pkt);
 
   total_read = 0;
+  eof = 0;
+  numpkts = 0;
   while (1)
     {
-      if (0 > (ret = sftp_write_pkt (p, soft_errors,
-                                     "14s84", SSH_FXP_READ, ++p->request_id,
-                                     handle, handle_len, total_read,
-                                     SFTP_RWSIZE)))
-        {
-          lim_free (0, handle);
-          return -1;
-        }
+      for (; ! eof && p->plreqs <= MIN(MAX_PL_REQS, numpkts + 1); p->plreqs++)
+	{
+	  ret = sftp_write_pkt (p, soft_errors, "14s84", SSH_FXP_READ,
+				++p->request_id, handle, handle_len,
+				total_read + SFTP_RWSIZE * p->plreqs,
+				SFTP_RWSIZE);
+	  if (0 > ret)
+	    {
+	      lim_free (0, handle);
+	      return -1;
+	    }
+	}
 
       if (0 > (i = sftp_server_response (p, &pkt, &pkt_len, SSH_FXP_DATA,
                                          soft_errors)))
@@ -694,12 +650,27 @@ sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
           return -1;
         }
 
+      p->plreqs--;
+      numpkts++;
+
       if (i == 0)
         {
+	  if (p->plreqs)
+	    {
+	      eof = 1;
+	      continue;
+	    }
+
           lim_free (0, pkt);
           break;
         }
 
+      if (eof)
+	{
+	  safe_printfmt(2, "EOF already reached, but the server sent data!\n");
+	  exit(2);
+	}
+
       len = READ_UINT4 (pkt + i);
       i += 4;
       if (len > pkt_len - i)
@@ -730,6 +701,12 @@ sftp_get (struct arch_pfs_sftp_session * p, int data_fd, t_uchar * path,
       lim_free (0, pkt);
     }
 
+  if (p->plreqs)
+    {
+      safe_printfmt (2, "sftp_client_get: Requests still in pipe-line!\n");
+      exit (2);
+    }
+
   if (0 > sftp_write_pkt (p, soft_errors,
                           "14s", SSH_FXP_CLOSE, ++p->request_id,
                           handle, handle_len))
@@ -773,7 +750,10 @@ pfs_file_contents (struct arch_pfs_session * p, t_uchar * path, int soft_errors)
   safe_unlink (tmp_path);
 
   if (0 > sftp_get (pfs, fd, path, soft_errors))
-    return 0;
+    {
+      safe_close(fd);
+      return 0;
+    }
 
   safe_lseek (fd, (off_t)0, SEEK_SET);
   answer = fd_contents (fd);
@@ -883,7 +863,7 @@ pfs_directory_files (struct arch_pfs_session * p, t_uchar * path,
           i += 4;
           if (len > pkt_len - i)
             {
-              if (!soft_errors)
+              if (! soft_errors)
                 {
                   safe_printfmt(2, "sftp_client_list: Packet too short(6): did we get garbage from server?\n");
                   lim_free (0, handle);
@@ -1000,7 +980,7 @@ static int pfs_put_file (struct arch_pfs_session *p, t_uchar * path,
   unsigned long long total_written;
   char *handle;
   char buf[SFTP_RWSIZE];
-  int len;
+  int len, eof;
 
   file = sftp_abs_path (pfs->cwd, path);
 
@@ -1038,27 +1018,36 @@ static int pfs_put_file (struct arch_pfs_session *p, t_uchar * path,
   lim_free (0, pkt);
 
   total_written = 0;
+  eof = 0;
   while (1)
     {
-      if (0 > (len = vu_read_retry (&errn, data_fd, buf, sizeof (buf))))
-        {
-          if (!soft_errors)
-            {
-              safe_printfmt (2, "Failed to read file to put()!\n");
-              exit (2);
-            }
-          lim_free (0, handle);
-          return -1;
-        }
-      if (0 > (ret = sftp_write_pkt (pfs, soft_errors,
-                                     "14s8s", SSH_FXP_WRITE, ++pfs->request_id,
-                                     handle, handle_len, total_written, buf,
-                                     len)))
-        {
-          lim_free (0, handle);
-          return -1;
-        }
+      for (; ! eof && pfs->plreqs <= MAX_PL_REQS; pfs->plreqs++)
+	{
+	  if (0 > (len = vu_read_retry (&errn, data_fd, buf, sizeof (buf))))
+	    {
+	      if (!soft_errors)
+		{
+		  safe_printfmt (2, "Failed to read file to put()!\n");
+		  exit (2);
+		}
+	      lim_free (0, handle);
+	      return -1;
+	    }
 
+	  ret = sftp_write_pkt (pfs, soft_errors, "14s8s", SSH_FXP_WRITE,
+				++pfs->request_id, handle, handle_len,
+				total_written + pfs->plreqs * sizeof (buf),
+				buf, len);
+	  if (0 > ret)
+	    {
+	      lim_free (0, handle);
+	      return -1;
+	    }
+	  
+	  if (len < sizeof (buf))
+	    eof = 1;
+	}
+      
       if (0 > (i = sftp_server_response (pfs, &pkt, &pkt_len, SSH_FXP_STATUS,
                                          soft_errors)))
         {
@@ -1067,10 +1056,7 @@ static int pfs_put_file (struct arch_pfs_session *p, t_uchar * path,
         }
 
       i = sftp_decode_status (pkt, pkt_len, i, soft_errors);
-
       lim_free (0, pkt);
-
-
       if (i == -1)
         {
           lim_free (0, handle);
@@ -1079,8 +1065,14 @@ static int pfs_put_file (struct arch_pfs_session *p, t_uchar * path,
 
       total_written += len;
 
-      if (len < sizeof (buf))
-        break;
+      if (! --pfs->plreqs)
+	break;
+    }
+
+  if (pfs->plreqs)
+    {
+      safe_printfmt (2, "pfs_put_file: Requests still in pipe-line!\n");
+      exit (2);
     }
 
   ret = sftp_write_pkt (pfs, soft_errors,
@@ -1408,6 +1400,75 @@ dirfold (t_uchar *dir)
   return dir;
 }
 
+extern int arch_pfs_sftp_parse_uri (t_uchar ** user, t_uchar ** hostname, t_uchar ** port, char ** path, t_uchar * uri)
+{
+  t_uchar * start;
+  t_uchar * stop;
+  t_uchar * host_part = 0;
+  t_uchar * at;
+  t_uchar * h0;
+  t_uchar * h1;
+  t_uchar * colon;
+
+  /* [user@]hostname[:port]/path
+   * ^    ^ ^       ^      ^
+   * |    | |       |      |
+   * |   at?|       |      stop[*]
+   * |      |       |	     0-byte of host_part
+   * |     h0       |
+   * |              |
+   * |              |
+   * |              |
+   * start[*]     colon?
+   * host_part    h1
+   * 
+   * 
+   * start and stop point to one copy of the string
+   * (the original uri which includes the /path).
+   * 
+   * host_part points to another copy of the string 
+   * (local to this block) which all the other pointers
+   * point into.   
+   * 
+   * at? and colon? are 0 if @ and : are not present.
+   * 
+   * The reason to make host_part is so that str_chr_index 
+   * searches are confined to just the host_part.
+   */
+
+
+  start = uri + sizeof ("sftp://") - 1;
+  stop = str_chr_index (start, '/');
+  if (!stop)
+    return -1;
+  *path = str_save (0, stop);
+
+  host_part = str_save_n (0, start, stop - start);
+
+  at = str_chr_index (host_part, '@');
+  if (at)
+    h0 = at + 1;
+  else
+    h0 = host_part;
+
+  colon = str_chr_index (h0, ':');
+  if (colon)
+    h1 = colon;
+  else
+    h1 = h0 + str_length (h0);
+
+  if (at)
+    *user = str_save_n (0, host_part, at - host_part);
+
+  if (colon)
+    *port = str_save (0, colon + 1);
+
+  *hostname = str_save_n (0, h0, h1 - h0);
+
+  lim_free (0, host_part);
+  return 0;
+}
+
 
 
 
