@@ -27,12 +27,13 @@ static void undo_callback (void * vfd, char * fmt, va_list ap);
 
 
 void
-arch_undo (int chatter_fd, t_uchar * dest_dir, int delete_changeset, t_uchar * tree_root, t_uchar * baseline_root, assoc_table inode_shortcut, int forward)
+arch_undo (int chatter_fd, t_uchar * dest_dir, int delete_changeset, t_uchar * tree_root,
+           t_uchar * baseline_root, assoc_table inode_shortcut, int forward,   int escape_classes)
 {
   struct arch_make_changeset_report make_report = {0, };
   struct arch_apply_changeset_report apply_report = {0, };
 
-  arch_make_changeset (&make_report, baseline_root, tree_root, dest_dir, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0);
+  arch_make_changeset (&make_report, baseline_root, tree_root, dest_dir, arch_unspecified_id_tagging, arch_inventory_unrecognized, 0, inode_shortcut, 0, escape_classes);
 
   arch_chatter (chatter_fd, "* reverting changes\n");
   if (chatter_fd >= 0)
@@ -41,7 +42,7 @@ arch_undo (int chatter_fd, t_uchar * dest_dir, int delete_changeset, t_uchar * t
       apply_report.thunk = (void *)(long)chatter_fd;
     }
 
-  arch_apply_changeset (&apply_report, dest_dir, tree_root, arch_unspecified_id_tagging, arch_inventory_unrecognized, 1, forward, 0, 0, 0);
+  arch_apply_changeset (&apply_report, dest_dir, tree_root, arch_unspecified_id_tagging, arch_inventory_unrecognized, 1, forward, 0, 0, 0, escape_classes);
 
   if (delete_changeset)
     rmrf_file (dest_dir);
